{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSTrackBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  extctrls;

const SliderThickness = 7;
      FBoundMin = 0;

type
  TTrackDirection = (tdUp, tdRight);
  TTimerMouse = record
                 mbtn : TMouseButton;
                 shst : TShiftState;
                 xmouse : Integer;
                 ymouse : Integer;
                end;
  TLCARSTrackBar = class(TGraphicControl)
  private
   FMinValue: Integer;
   FMaxValue: Integer;
   FCurValue: Integer;
   FShowText: Boolean;
   FBoundsThickness : Integer;
   FTrackBarColor : TColor;
   FTrackDirection : TTrackDirection;
   FBackColor: TColor;
   FTextFrequency : Integer;
   FTickFrequency : Integer;
   FCriticalValue : Integer;
   FZeroPercent : Integer;
   FHundredPercent : Integer;
   FTextPlace : Integer;
   FBarPlace : Integer;
   FPercentPlace : Integer;
   FPositionPlace : Integer;
   FProgressToShow : Integer;
   FOldToShow : Integer;
   FTextWidth : Integer;
   FTextHeight : Integer;
   FTextPosX : Integer;
   FTextPosY : Integer;
   FSliderGrabbed : Boolean;
   FOldMarkerPos : Integer;
   FMouseIsPressed : Boolean;
   FTimerMouse : TTimerMouse;
   FBack : TBitmap;
   FSliderTimer : TTimer;
   FOnChange : TNotifyEvent;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   procedure SetShowText(Value: Boolean);
   procedure SetMinValue(Value: Integer);
   procedure SetMaxValue(Value: Integer);
   procedure SetPosition(Value: Integer);
   procedure SetTrackBarColor(value : TColor);
   procedure SetTrackDirection(Value : TTrackDirection);
   procedure SetTextFrequency(Value : Integer);
   procedure SetTickFrequency(Value : Integer);
   procedure SetCriticalValue(Value : Integer);
   function CalculateNewPosition : Boolean;
   function OverSlider(x, y : Integer) : Boolean;
   procedure MoveSlider(x, y : Integer);
   procedure UpdatePosition(x, y : Integer);
   procedure SliderTimerTimer(Sender: TObject);
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
   procedure Paint; override;
   procedure Change ; dynamic;
   procedure MouseEnter; dynamic;
   procedure MouseLeave; dynamic;
   procedure MouseDown(Button:TMouseButton;
             Shift:TShiftState;X,Y:Integer);override;
   procedure MouseMove(Shift:TShiftState;
             X,Y:Integer);override;
   procedure MouseUp(Button:TMouseButton;
             Shift:TShiftState;X,Y:Integer);override;
   procedure PaintSlider;
   procedure DrawTicks;
  public
   property MouseInControl: Boolean read FMouseInControl;
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure AddPosition(Value: Integer);
  published
   property Align;
   property Enabled;
   property Font;
   property TrackBarDirection : TTrackDirection read FTrackDirection
    write SetTrackDirection;
   property TrackBarColor : TColor read FTrackBarColor write SetTrackBarColor;
   property MinValue: Integer read FMinValue write SetMinValue default 0;
   property MaxValue: Integer read FMaxValue write SetMaxValue default 100;
   property ParentColor;
   property ParentFont default true;
   property ParentShowHint;
   property Position: Integer read FCurValue write SetPosition default 0;
   property ShowHint;
   property ShowText: Boolean read FShowText write SetShowText default True;
   property Visible;
   property TextFrequency : Integer read FTextFrequency
    write SetTextFrequency;
   property TickFrequency : Integer read FTickFrequency
    write SetTickFrequency;
   property CriticalValue : Integer read FCriticalValue
    write SetCriticalValue;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnChange: TNotifyEvent read FOnChange write FOnChange;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

implementation

constructor TLCARSTrackBar.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FPositionPlace := 1;
 Width := 50;
 Height := 200;
 FMinValue := 0;
 FMaxValue := 100;
 Position := 0;
 FShowText := True;
 FTextFrequency := 10;
 FTickFrequency := 5;
 TrackbarColor := clyellow;
 ControlStyle := ControlStyle + [csOpaque];
 {Canvas.Brush.Style := bsClear;}
 FBack := TBitmap.Create;

end;

destructor TLCARSTrackBar.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSTrackBar.PaintSlider;
begin
 FBack.Canvas.Brush.Color := FTrackBarColor;
 FBack.Canvas.Pen.Color := FTrackBarColor;
 if FTrackDirection = tdRight then
  begin
   FBack.Canvas.Rectangle(FZeroPercent + FProgressToShow - (SliderThickness div 2), FBoundsThickness,
                          FZeroPercent + FProgressToShow + (SliderThickness div 2), FBack.Height - FBoundsThickness);
  end
 else if FTrackDirection = tdUp then
  begin
   FBack.Canvas.Rectangle(FBoundsThickness, FBack.Height - FZeroPercent - FProgressToShow - (SliderThickness div 2),
                          FBack.Width - FBoundsThickness, FBack.Height - FZeroPercent - FProgressToShow + (SliderThickness div 2));
  end;
end;

procedure TLCARSTrackBar.Paint;
var VisibleRect, BackRect, BoundRect : TRect;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0,0, FBack.width, FBack.height);
 FBoundsThickness := 2;
 if FShowText then
  begin
   if FTrackDirection = tdRight then
    FTextPlace := FBack.Canvas.TextWidth('88888') + 4
   else if FTrackDirection = tdUp then
    FTextPlace := FBack.Canvas.TextHeight('88888') + 4;
   FTextWidth := FBack.Canvas.TextWidth(InttoStr(Position));
   FTextHeight := FBack.Canvas.TextHeight(InttoStr(Position));
  end
 else
  FTextPlace := 0;
 if FTrackDirection = tdRight then
  FBarPlace := FBack.Width - FTextPlace
 else if FTrackDirection = tdUp then
  FBarPlace := FBack.Height - FTextPlace;
 FPositionPlace := MaxValue - MinValue;
 FZeroPercent := FBoundsThickness + (SliderThickness div 2) + 1;
 FHundredPercent := FBarPlace - FBoundsThickness - (SliderThickness div 2) - 1;
 FPercentPlace := FHundredPercent - FZeroPercent;
 FBack.Canvas.Brush.Color := FTrackBarColor;
 FBack.Canvas.Pen.Color := FTrackBarColor;
 if FTrackDirection = tdRight then
  begin
   CalculateNewPosition;
   DrawTicks;
   PaintSlider;
   BoundRect := Rect(0, 0, FBarPlace, FBack.Height);
   FBack.Canvas.FrameRect(BoundRect);
   BoundRect := Rect(1, 1, FBarPlace-1, FBack.Height-1);
   FBack.Canvas.FrameRect(BoundRect);
   if FShowText then
    begin
     FBack.Canvas.Brush.Color := FBackColor;
     FBack.Canvas.Pen.Color := FBackColor;
     FTextPosX := (FBack.Width - FTextPlace) + (FTextPlace div 2) - (FTextWidth div 2);
     FTextPosY := (FBack.Height div 2) - (FTextHeight div 2);
     FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Position));
    end;
  end
 else if FTrackDirection = tdUp then
  begin
   CalculateNewPosition;
   DrawTicks;
   PaintSlider;
   BoundRect := Rect(0, FBack.Height - FBarPlace, FBack.Width, FBack.Height);
   FBack.Canvas.FrameRect(BoundRect);
   BoundRect := Rect(1, FBack.Height - FBarPlace+1, FBack.Width-1, FBack.Height-1);
   FBack.Canvas.FrameRect(BoundRect);
   if FShowText then
    begin
     FBack.Canvas.Brush.Color := FBackColor;
     FBack.Canvas.Pen.Color := FBackColor;
     FTextPosX := (FBack.Width div 2) - (FTextWidth div 2);
     FTextPosY := (FTextPlace div 2) - (FTextHeight div 2);
     FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Position));
    end;
  end;
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, Fback.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSTrackBar.DrawTicks;
var DrawValue, DrawPos, count : Integer;
    Factor : Real;
begin
 DrawValue := FMinValue;
 while DrawValue <= FMaxValue do
  begin
   Factor := FPercentPlace / FPositionPlace;
   DrawPos := Round(Factor * (DrawValue - FMinValue));
   if FTrackDirection = tdRight then
    begin
     for count := -1 to 1 do
      begin
       FBack.Canvas.MoveTo(FZeroPercent + DrawPos + count, FBoundsThickness);
       FBack.Canvas.LineTo(FZeroPercent + DrawPos + count, FBoundsThickness + 10);
       FBack.Canvas.MoveTo(FZeroPercent + DrawPos + count, FBack.Height - FBoundsThickness - 10);
       FBack.Canvas.LineTo(FZeroPercent + DrawPos + count, FBack.Height - FBoundsThickness);
      end;
     if (DrawValue <> MinValue) and (Drawvalue <> MaxValue) then
      begin
       FBack.Canvas.Brush.Color := FBackColor;
       FBack.Canvas.Pen.Color := FBackColor;
       FBack.Canvas.TextOut(FZeroPercent + DrawPos - (FBack.Canvas.TextWidth(InttoStr(DrawValue)) div 2),
                            (FBack.Height div 2) - (FBack.Canvas.TextHeight(InttoStr(DrawValue)) div 2),
                            inttostr(DrawValue));
       FBack.Canvas.Brush.Color := FTrackBarColor;
       FBack.Canvas.Pen.Color := FTrackBarColor;
      end;
    end
   else if FTrackDirection = tdUp then
    begin
     for count := -1 to 1 do
      begin
       FBack.Canvas.MoveTo(FBoundsThickness, FBack.Height - FZeroPercent - DrawPos + count);
       FBack.Canvas.LineTo(FBoundsThickness + 10 , FBack.Height - FZeroPercent - DrawPos + count);
       FBack.Canvas.MoveTo(FBack.Width - FBoundsThickness - 10, FBack.Height - FZeroPercent - DrawPos + count);
       FBack.Canvas.LineTo(FBack.Width - FBoundsThickness, FBack.Height - FZeroPercent - DrawPos + count);
      end;
     if (DrawValue <> MinValue) and (Drawvalue <> MaxValue) then
      begin
       FBack.Canvas.Brush.Color := FBackColor;
       FBack.Canvas.Pen.Color := FBackColor;
       FBack.Canvas.TextOut((FBack.Width div 2) - (FBack.Canvas.TextWidth(InttoStr(DrawValue)) div 2),
                            FBack.Height - FZeroPercent - DrawPos - (FBack.Canvas.TextHeight(InttoStr(DrawValue)) div 2),
                            inttostr(DrawValue));
       FBack.Canvas.Brush.Color := FTrackBarColor;
       FBack.Canvas.Pen.Color := FTrackBarColor;
      end;
    end;
   DrawValue := DrawValue + FTextFrequency;
  end;
  DrawValue := FMaxValue + FMinValue;
  Factor := FPercentPlace / FPositionPlace;
  DrawPos := Round(Factor * (DrawValue - FMinValue));
  if FTrackDirection = tdRight then
   begin
    for count := -1 to 1 do
     begin
      FBack.Canvas.MoveTo(FZeroPercent + DrawPos + count, FBoundsThickness);
      FBack.Canvas.LineTo(FZeroPercent + DrawPos + count, FBoundsThickness + 10);
      FBack.Canvas.MoveTo(FZeroPercent + DrawPos + count, FBack.Height - FBoundsThickness - 10);
      FBack.Canvas.LineTo(FZeroPercent + DrawPos + count, FBack.Height - FBoundsThickness);
     end;
   end
  else if FTrackDirection = tdUp then
   begin
    for count := -1 to 1 do
     begin
      FBack.Canvas.MoveTo(FBoundsThickness, FBack.Height - FZeroPercent - DrawPos + count);
      FBack.Canvas.LineTo(FBoundsThickness + 10 , FBack.Height - FZeroPercent - DrawPos + count);
      FBack.Canvas.MoveTo(FBack.Width - FBoundsThickness - 10, FBack.Height - FZeroPercent - DrawPos + count);
      FBack.Canvas.LineTo(FBack.Width - FBoundsThickness, FBack.Height - FZeroPercent - DrawPos + count);
     end;
   end;
 DrawValue := FMinValue;
 while DrawValue <= FMaxValue do
  begin
   Factor := FPercentPlace / FPositionPlace;
   DrawPos := Round(Factor * (DrawValue - FMinValue));
   if FTrackDirection = tdRight then
    begin
     FBack.Canvas.MoveTo(FZeroPercent + DrawPos, FBoundsThickness);
     FBack.Canvas.LineTo(FZeroPercent + DrawPos, FBoundsThickness + 6);
     FBack.Canvas.MoveTo(FZeroPercent + DrawPos, FBack.Height - FBoundsThickness - 6);
     FBack.Canvas.LineTo(FZeroPercent + DrawPos, FBack.Height - FBoundsThickness);
    end
   else if FTrackDirection = tdUp then
    begin
     FBack.Canvas.MoveTo(FBoundsThickness, FBack.Height - FZeroPercent - DrawPos);
     FBack.Canvas.LineTo(FBoundsThickness + 6 , FBack.Height - FZeroPercent - DrawPos);
     FBack.Canvas.MoveTo(FBack.Width - FBoundsThickness - 6, FBack.Height - FZeroPercent - DrawPos);
     FBack.Canvas.LineTo(FBack.Width - FBoundsThickness, FBack.Height - FZeroPercent - DrawPos);
    end;
   DrawValue := DrawValue + FTickFrequency;
  end;
 DrawValue := FCriticalValue;
 Factor := FPercentPlace / FPositionPlace;
 DrawPos := Round(Factor * (DrawValue - FMinValue));
 if FTrackDirection = tdRight then
  begin
   FBack.Canvas.Rectangle(FZeroPercent + DrawPos, FBoundsThickness,
                          FHundredPercent, FBoundsThickness + 4);
   FBack.Canvas.Rectangle(FZeroPercent + DrawPos, FBack.Height - FBoundsThickness - 4,
                       FHundredPercent, FBack.Height - FBoundsThickness);
  end
 else if FTrackDirection = tdUp then
  begin
   FBack.Canvas.Rectangle(FBoundsThickness, FBack.Height - FZeroPercent - DrawPos,
                          FBoundsThickness + 4, FBack.Height - FHundredPercent);
   FBack.Canvas.Rectangle(FBack.Width - FBoundsThickness - 4, FBack.Height - FZeroPercent - DrawPos,
                          FBack.Width - FBoundsThickness, FBack.Height - FHundredPercent);
  end;
end;

procedure TLCARSTrackBar.UpdatePosition(x, y : Integer);
var Factor : real;
begin
 Factor := FPercentPlace / FPositionPlace;
 if FTrackDirection = tdRight then
  FProgressToShow := x - FZeroPercent
 else if FTrackDirection = tdUp then
  FProgressToShow := FBack.Height - FZeroPercent - y;
 if FProgressToShow < 0 then FProgressToShow := 0;
 if FProgressToShow > FHundredPercent then FProgressToShow := FHundredPercent;
 Position := Round(FProgressToShow / Factor) + FMinValue;
end;

function TLCARSTrackBar.CalculateNewPosition : boolean;
var Factor : real;
begin
 FOldToShow := FProgressToShow;
 Factor := FPercentPlace / FPositionPlace;
 FProgressToShow := Round(Factor * (FCurValue - FMinValue));
 if not FSliderGrabbed then
  begin
   if FTrackDirection = tdRight then
    FOldMarkerPos := FZeroPercent + FProgressToShow
   else if FTrackDirection = tdUp then
    FOldMarkerPos := FBack.Height - FZeroPercent - FProgressToShow;
 end;
 if FProgressToShow = FOldToShow then
  CalculateNewPosition := false
 else
  CalculateNewPosition := true;
end;

function TLCARSTrackBar.OverSlider(x, y : Integer) : Boolean;
begin
 OverSlider := false;
 if FTrackDirection = tdRight then
  begin
   if (x >= FZeroPercent + FProgressToShow - (SliderThickness div 2)) and
      (x <= FZeroPercent + FProgressToShow + (SliderThickness div 2)) then
    OverSlider := true;
  end
 else if FTrackDirection = tdUp then
  begin
   if (y >= FBack.Height - FZeroPercent - FProgressToShow - (SliderThickness div 2)) and
      (y <= FBack.Height - FZeroPercent - FProgressToShow + (SliderThickness div 2)) then
    OverSlider := true;
  end;
end;

procedure TLCARSTrackBar.MoveSlider(x, y : Integer);
begin
 if FTrackDirection = tdRight then
  begin
   if x < FOldMarkerPos then
    begin
     AddPosition(-1);
    end
   else if x > FOldMarkerPos then
    begin
     AddPosition(1);
    end;
  end
 else if FTrackDirection = tdUp then
  begin
   if y > FOldMarkerPos then
    begin
     AddPosition(-1);
    end
   else if y < FOldMarkerPos then
    begin
     AddPosition(1);
    end;
  end;
end;

procedure TLCARSTrackBar.MouseDown(Button:TMouseButton;
       Shift:TShiftState;X,Y:Integer);
begin
 inherited MouseDown(Button,Shift,X,Y);
 FTimerMouse.mbtn := Button;
 FTimerMouse.shst := Shift;
 FTimerMouse.xmouse := x;
 FTimerMouse.ymouse := y;
 FMouseIsPressed := true;
 if (Button = mbLeft) and (OverSlider(x, y)) then
  FSliderGrabbed := true;
 if (Button = mbLeft) and not OverSlider(x, y) and
    (FMouseIsPressed = true) then
  begin
   if FSliderTimer = nil then
    begin
     FSliderTimer := TTimer.Create(self);
     FSliderTimer.OnTimer := SliderTimerTimer;
     FSliderTimer.Interval := 500;
    end
   else
    begin
     FSliderTimer.OnTimer := SliderTimerTimer;
     FSliderTimer.Interval := 100;
    end;
   FSliderTimer.Enabled := true;
   MoveSlider(x, y);
  end;
end;

Procedure TLCARSTrackBar.MouseUp(Button:TMouseButton;
       Shift:TShiftState;X,Y:Integer);
begin
 FMouseIsPressed := false;
 if (Button = mbLeft) and (FSliderGrabbed) then
  FSliderGrabbed := False;
 if FSliderTimer <> nil then
  begin
   FSliderTimer.Enabled := false;
   FSliderTimer := nil;
  end;
 inherited MouseUp(Button,Shift,X,Y);
end;

procedure TLCARSTrackBar.MouseMove(Shift:TShiftState;X,Y:Integer);
begin
 FTimerMouse.xmouse := x;
 FTimerMouse.ymouse := y;
 if FSliderGrabbed then
  begin
   UpdatePosition(x, y);
   if FTrackDirection = tdRight then
    FOldMarkerPos := x
   else if FTrackDirection = tdUp then
    FOldMarkerPos := y;
  end;
 if (ssLeft in Shift) and (OverSlider(x, y)) then
  begin
   FSliderGrabbed := true;
  end
 else FSliderGrabbed := false;
end;

procedure TLCARSTrackBar.SliderTimerTimer(Sender: TObject);
begin
 if FMouseIsPressed and MouseCapture then
  begin
    try
     MouseDown(FTimerMouse.mbtn, FTimerMouse.shst,
               FTimerMouse.xmouse, FTimerMouse.ymouse);
    except
     FSliderTimer.Enabled := False;
     raise;
    end;
  end;
end;

procedure TLCARSTrackBar.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TLCARSTrackBar.SetShowText(Value: Boolean);
begin
  if Value <> FShowText then
  begin
    FShowText := Value;
    Repaint;
  end;
end;

procedure TLCARSTrackBar.SetTrackBarColor(Value: TColor);
begin
  if Value <> FTrackBarColor then
  begin
    FTrackBarColor := Value;
    Repaint;
  end;
end;

procedure TLCARSTrackBar.SetTrackDirection(Value: TTrackDirection);
begin
 if Value <> FTrackDirection then
  begin
   FTrackDirection := Value;
   Repaint;
  end;
end;

procedure TLCARSTrackBar.SetTextFrequency(Value: Integer);
begin
 if Value < 1 then Value := 1;
 if Value <> FTextFrequency then
  begin
   FTextFrequency := Value;
   Repaint;
  end;
end;

procedure TLCARSTrackBar.SetCriticalValue(Value: Integer);
begin
 if Value <> FCriticalValue then
  begin
   FCriticalValue := Value;
   Repaint;
  end;
end;

procedure TLCARSTrackBar.SetTickFrequency(Value: Integer);
begin
 if Value < 1 then Value := 1;
 if Value <> FTickFrequency then
  begin
   FTickFrequency := Value;
   Repaint;
  end;
end;

procedure TLCARSTrackBar.SetMinValue(Value: Integer);
begin
  if Value <> FMinValue then
  begin
    if Value > FMaxValue then
     FMinValue := FCurValue
    else
     FMinValue := Value;
    if FCurValue < Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TLCARSTrackBar.SetMaxValue(Value: Integer);
begin
  if Value <> FMaxValue then
  begin
    if Value < FMinValue then
     FMaxValue := FCurValue
    else
     FMaxValue := Value;
    if FCurValue > Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TLCARSTrackBar.SetPosition(Value: Integer);
begin
 if Value < FMinValue then
  Value := FMinValue
 else if Value > FMaxValue then
  Value := FMaxValue;
 if FCurValue <> Value then
  begin
   FCurValue := Value;
   Change;
  end;
 Repaint;
end;

procedure TLCARSTrackBar.AddPosition(Value: Integer);
begin
  Position := FCurValue + Value;
end;

procedure TLCARSTrackBar.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSTrackBar.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSTrackBar.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSTrackBar.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

end.
