{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSMLButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TLCARSMLButton = class(TGraphicControl)
  private
   FRoundLeft : boolean;
   FRoundRight : boolean;
   FEnabledColor : TColor;
   FDisabledColor : TColor;
   FBack : TBitmap;
   FBackColor : TColor;
   FCaption : TCaption;
   FCaptionWidth : integer;
   FCaptionHeight : integer;
   FCaptionPosX : integer;
   FCaptionPosY : integer;
   FRadiusLeft : integer;
   FRadiusRight : integer;
   FHyphens : String;
   FPCHyphens : PChar;
   FTempCaption : PChar;
   P : PChar;
   FLetterList : TStringList;
   FCaptionPartList : TStringList;
   FLineList : TStringList;
   FCaptionPart : string;
   FLine : String;
   FCaptionPlace : integer;
   FModalResult: TModalResult;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   procedure SetCaption(value : TCaption);
   procedure SetRoundLeft(value : boolean);
   procedure SetRoundRight(value : boolean);
   procedure SetEnabledColor(value : TColor);
   procedure SetDisabledColor(value : TColor);
   procedure SetHyphens(value : string);
   procedure PrepareCaption;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Paint; override;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
  public
    property MouseInControl: Boolean read FMouseInControl;
    constructor Create(AOwner : TComponent) ; override;
    destructor Destroy; override;
    procedure Click; override;
  published
   property Anchors;
   property Constraints;
   property Parent;
   property Width default 129;
   property Height default 41;
   property Visible default true;
   property Enabled default true;
   property Font;
   property ParentFont default true;
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property ModalResult: TModalResult read FModalResult write FModalResult default 0;
   property Caption : TCaption read FCaption
    write SetCaption;
   property RoundLeft : boolean read FRoundLeft
    write SetRoundLeft default true;
   property RoundRight : boolean read FRoundRight
    write SetRoundRight default true;
   property EnabledColor : TColor read FEnabledColor
    write SetEnabledColor default clblue;
   property DisabledColor : TColor read FDisabledColor
    write SetDisabledColor default clgray;
   property Hyphens : String read FHyphens write SetHyphens;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSMLButton.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 129;
 Height := 41;
 EnabledColor := clblue;
 DisabledColor := clgray;
 RoundLeft := true;
 RoundRight := true;
 Visible := true;
 FRadiusLeft := Height div 2;
 FRadiusRight := Height div 2;
 Canvas.Brush.Style := bsClear;
 ControlStyle:=ControlStyle+[csOpaque];
 FBack := TBitmap.Create;
 FLetterList := TStringList.Create;
 FLetterList.Sorted := false;
 FCaptionPartList := TStringList.Create;
 FCaptionPartList.Sorted := false;
 FLineList := TStringList.Create;
 FLineList.Sorted := false;
 ParentFont := true;
end;

destructor TLCARSMLButton.Destroy;
begin
 FBack.Free;
 FLetterList.Free;
 FCaptionPartList.Free;
 FLineList.Free;
 inherited Destroy;
end;

procedure TLCARSMLButton.Click;
var
  Form: TForm{TCustomForm};
begin
  Form := TForm(GetParentForm(Self));
  if Form <> nil then Form.ModalResult := ModalResult;
  inherited Click;
end;

procedure TLCARSMLButton.Paint;
var VisibleRect, BackRect : TRect;
    LineCounter : integer;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 PrepareCaption;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FCaptionWidth := 0;
 if FLineList.Count > 0 then
  begin
   for LineCounter := 0 to FLineList.Count-1 do
    begin
     if FBack.Canvas.TextWidth(FlineList.Strings[LineCounter]) > FCaptionWidth then
      FCaptionWidth := FBack.Canvas.TextWidth(FlineList.Strings[LineCounter]);
    end;
  end;
 FCaptionHeight := FLineList.Count * FBack.Canvas.TextHeight(FCaption);
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0, 0, FBack.Width, FBack.Height);
 if Enabled then
  begin
   FBack.Canvas.Brush.Color := EnabledColor;
   FBack.Canvas.Pen.Color := EnabledColor;
  end
 else
  begin
   FBack.Canvas.Brush.Color := DisabledColor;
   FBack.Canvas.Pen.Color := DisabledColor;
  end;
 FBack.Canvas.RoundRect(0,0, FBack.width, FBack.height,
                        FBack.height,FBack.height);
 if RoundLeft = false then
  FBack.Canvas.Rectangle(0, 0, FBack.Height div 2 + 1, FBack.Height);
 if RoundRight = false then
  FBack.Canvas.Rectangle(FBack.Width - (FBack.Height div 2 + 1), 0, FBack.Width, FBack.Height);
 FCaptionPosX := (FBack.Width div 2) - (FCaptionWidth div 2);
 FCaptionPosY := (FBack.Height div 2) - (FCaptionHeight div 2);
 if FLineList.Count > 0 then
  begin
   for LineCounter := 0 to FlineList.Count - 1 do
    FBack.Canvas.TextOut(FCaptionPosX,
    FCaptionPosY + LineCounter * FBack.Canvas.TextHeight(FCaption),
     FLineList.Strings[LineCounter]);
  end;
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, Fback.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSMLButton.PrepareCaption;
var FCounter : integer;
begin
 FLetterList.Clear;
 FCaptionPartList.Clear;
 FLineList.Clear;
 FCaptionPart := '';
 FPCHyphens := nil;
 FTempCaption := nil;
 P := nil;
 if FHyphens <> '' then FPCHyphens := PChar(FHyphens);
 P := StrPos(FPCHyphens, ' ');
 if P = nil then FHyphens := fHyphens + ' ';
 FPCHyphens := PChar(FHyphens);
 FTempCaption := PChar(FCaption + ' ');
 if FRoundLeft = false then FRadiusLeft := 0
 else FRadiusLeft := height div 2;
 if FRoundRight = false then FRadiusRight := 0
 else FRadiusRight := height div 2;
 if FCaption <> '' then
  begin
   FCaptionPart := '';
   for FCounter := 0 to StrLen(FTempCaption)-1 do
    FLetterList.Add(FTempCaption[FCounter]);
   for FCounter := 0 to FletterList.Count-1 do
    begin
     FCaptionPart := FCaptionPart + FLetterList.Strings[FCounter];
     P := StrPos(FPCHyphens, PChar(FLetterList.Strings[FCounter]));
     if P <> nil then
      begin
       FCaptionPartList.Add(FCaptionPart);
       FCaptionPart := '';
      end;
    end;
  end;
 FCaptionPlace := Width - FRadiusLeft - FRadiusRight;
 FLine := '';
 if FCaptionPartList.Count > 0 then
  begin
   if FCaptionPartList.Count = 1 then
    FLineList.Add(FCaptionPartList.Strings[0])
   else if FCaptionPartList.Count > 1 then
    begin
     for FCounter := 0 to FCaptionPartList.Count - 1 do
      begin
       if FBack.Canvas.TextWidth(FLine + FCaptionPartList.Strings[FCounter]) > FCaptionPlace then
        begin
         FLineList.Add(Fline);
         Fline := '';
        end;
       Fline := Fline + FCaptionPartList.Strings[FCounter];
      end;
     FLineList.Add(Fline);
     Fline := '';
    end;
  end
 else if FCaptionPartList.Count = 0 then
  FLineList.Add('');
end;

procedure TLCARSMLButton.SetCaption(value : TCaption);
begin
 if FCaption <> value then
  begin
   FCaption := value;
   Refresh;
  end;
end;

procedure TLCARSMLButton.SetRoundLeft(value : boolean);
begin
 if FRoundLeft <> value then
  begin
   FRoundLeft := value;
   Refresh;
  end;
end;

procedure TLCARSMLButton.SetRoundRight(value : boolean);
begin
 if FRoundRight <> value then
  begin
   FRoundRight := value;
   Refresh;
  end;
end;

procedure TLCARSMLButton.SetHyphens(value : string);
begin
 if FHyphens <> value then
  begin
   FHyphens := value;
   Refresh;
  end;
end;

procedure TLCARSMLButton.SetEnabledColor(value : TColor);
begin
 if FEnabledColor <> value then
  begin
   FEnabledColor := value;
   Refresh;
  end;
end;

procedure TLCARSMLButton.SetDisabledColor(value : TColor);
begin
 if FDisabledColor <> value then
  begin
   FDisabledColor := value;
   Refresh;
  end;
end;

procedure TLCARSMLButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSMLButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSMLButton.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSMLButton.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;


procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSMLButton]);
end;

end.